//
//  Audio_ToolController.m
//  Audio Tool
//
//  Created by John Geelen on 9/15/11.
//  Copyright 2011 JG Electronics. All rights reserved.
//	Remarks, reports to: www.jgelectronics.nl.
//
//	Derived from Apple's example program DefaultOutputUnit.
//	The application icon is from Vincent Garnier http://benjigarner.deviantart.com/art/Fruity-Apples-80805804
//
//	Cocoa, OSX10.6 and XCODE 4.0.1
//
/*
 Disclaimer: IMPORTANT: 
 This JG Electronics software is supplied to you by JG Electronics as FREE AND OPEN SOURCE SOFTWARE. 
 
 The JG Electronics Software is provided by JG Electronics on an "AS IS" basis.
 JG ELECTRONICS MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE JG ELECTRONICS SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL JG ELECTRONICS BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE JG ELECTRONICS SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF JG ELECTRONICS HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

#import "Audio_ToolController.h"


@implementation Audio_ToolController

	#pragma mark -
	#pragma mark === Accessor Methods ===
//	*********************************************************************************************************************************
//	slider values
	@synthesize freqFlt, panFlt, amplitudeFlt;

	#pragma mark -
	#pragma mark === Internal Methods ===
//	*********************************************************************************************************************************
- (void)awakeFromNib {
	[self acCalcDisplayData];
}

- (id)init {
	if((self = [super init]) == nil)
		return nil;
	
	gAudioToolModelPtr = [[Audio_ToolModel alloc] init];
	phase = 0;
	[self setFreqFlt:1000];
	[self setPanFlt:0.5];
	[self setAmplitudeFlt:0.25];
	
//	key value observing
	[self addObserver:self forKeyPath:@"freqFlt" options:NSKeyValueObservingOptionOld context:nil];
	[self addObserver:self forKeyPath:@"amplitudeFlt" options:NSKeyValueObservingOptionOld context:nil];
	[self addObserver:self forKeyPath:@"panFlt" options:NSKeyValueObservingOptionOld context:nil];
	
	return self;
}

- (void)dealloc {
	[gAudioToolModelPtr release];
	
	[super dealloc];
}

	#pragma mark -
	#pragma mark === Key Value Observing Methods ===
//	*********************************************************************************************************************************
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
//	NSLog(@"observeValueForKeyPath");
	if([keyPath compare:@"freqFlt"] == NSOrderedSame) {
		[gAudioToolModelPtr amSetFrequency:[self freqFlt]];
		[self acCalcDisplayData];
	}
	if([keyPath compare:@"panFlt"] == NSOrderedSame) {
		[gAudioToolModelPtr amSetPan:[self panFlt]];
	}
	if([keyPath compare:@"amplitudeFlt"] == NSOrderedSame) {
	/*	if([waveForm selectedRow] == 2) {
			[gAudioToolModelPtr amSetAmpl:[self amplitudeFlt]];
			[gAudioToolModelPtr amSetamplitude:1.0];
		} else {
			[gAudioToolModelPtr amSetamplitude:[self amplitudeFlt]];
		}
	 */
		[gAudioToolModelPtr amSetamplitude:[self amplitudeFlt]];
		[self acCalcDisplayData];
	}
}

	#pragma mark -
	#pragma mark === Action Methods ===
//	*********************************************************************************************************************************
- (IBAction)acReadButtons:(id)sender {
//	NSLog(@"acReadButtons");
	switch ([sender tag])
	{
		case 0:
			[self acCalcDisplayData];
			break;
		case 1:
			[gAudioToolModelPtr amSetamplitude:[self amplitudeFlt]];	//	restore fadervalue to amplitude
			[gAudioToolModelPtr StartAudio];
			[acStartButton setEnabled:NO];
			[acStopButton setEnabled:YES];
			break;
		case 2:
			[gAudioToolModelPtr amSetamplitude:0.0];	//	to suppress click sound when switching off audio unit
			[gAudioToolModelPtr StopAudio];
			[acStartButton setEnabled:YES];
			[acStopButton setEnabled:NO];
		break;
	}
}


- (void)acCalcDisplayData {
	UInt32			frame;
	int				samples = 512;
	Float32			nextSample;
	float			phaseIncrement = 2.0 * M_PI * [self freqFlt] / 48000.0;
//	NSLog(@"phaseIncrement= %f", phaseIncrement);
	float			dt = phaseIncrement / (2.0 * M_PI * [self freqFlt]);
	float			dt_quarterPeriod = M_PI_2 / (2.0 * M_PI * [self freqFlt]);
//	NSLog(@"dt_quarterPeriod= %f", dt_quarterPeriod);
	float			tanPhi;
//	NSLog(@"tanPhi = %f", tanPhi);
	
	switch([waveForm selectedRow])	{
		case 0:		//	sine wave
			for (frame = 0; frame < samples; ++frame) {
				nextSample = sin(phase) * [self amplitudeFlt];
				buffer[frame] = nextSample;
//				outA[frame] = nextSample * pan;				//	left channel
//				outB[frame] = nextSample * (1.0 - pan);		//	right channel
				phase = phase + phaseIncrement;
				if (phase >= 2.0 * M_PI) {
					phase = phase - 2.0 * M_PI;
				}
			}
			[gAudioToolModelPtr amSetamplitude:[self amplitudeFlt]];
			[gAudioToolModelPtr amSetWaveForm:0];
			break;
			
		case 1:		//	square wave
			for (frame = 0; frame < samples; ++frame) {
				if(phase <= M_PI) {
					nextSample = [self amplitudeFlt];
				} else {
					nextSample = -[self amplitudeFlt];
				}
			//	or you can write
			//	nextSample = (phase <= M_PI) ? [self amplitudeFlt] : -[self amplitudeFlt];
				buffer[frame] = nextSample;
//				outA[frame] = nextSample * pan;				//	left channel
//				outB[frame] = nextSample * (1.0 - pan);		//	right channel
				phase = phase + phaseIncrement;
				if (phase >= 2.0 * M_PI) {
					phase = phase - 2.0 * M_PI;
				}
			}
			[gAudioToolModelPtr amSetamplitude:[self amplitudeFlt]];
			[gAudioToolModelPtr amSetWaveForm:1];
			break;
			
		case 2:		//	triangle wave version with 512 multiplications less then the case 3 version
			tanPhi = [self amplitudeFlt] / dt_quarterPeriod;
			for ( frame = 0; frame < samples; ++frame ) {
				dt = phase / (2.0 * M_PI * [self freqFlt]);
				if(phase <= M_PI_2) {
					nextSample = tanPhi * dt;
				}
				if((phase > M_PI_2) && (phase <= M_PI)) {
					nextSample = tanPhi * (2 * dt_quarterPeriod - dt);
				}
				if((phase > M_PI) && (phase <= (M_PI + M_PI_2))) {
					nextSample = - (tanPhi * (dt - 2 * dt_quarterPeriod));
				}
				if((phase > (M_PI + M_PI_2)) && (phase <= (2 * M_PI))) {
					nextSample = - (tanPhi * (4 * dt_quarterPeriod - dt));
				}
				buffer[frame] = nextSample;					//	no multiplication with [self amplitudeFlt]
//				outA[frame] = nextSample * pan;				//	left channel
//				outB[frame] = nextSample * (1.0 - pan);		//	right channel
				phase = phase + phaseIncrement;
				if (phase >= 2.0 * M_PI) {
					phase = phase - 2.0 * M_PI;
				}
			}
			[gAudioToolModelPtr amSetamplitude:[self amplitudeFlt]];
			[gAudioToolModelPtr amSetWaveForm:2];
			break;
			
		case 3:		//	triangle wave version with 512 extra multiplications
			tanPhi = 1 / dt_quarterPeriod;
			for ( frame = 0; frame < samples; ++frame ) {
				dt = phase / (2.0 * M_PI * [self freqFlt]);
				if(phase <= M_PI_2) {
					nextSample = tanPhi * dt;
				}
				if((phase > M_PI_2) && (phase <= M_PI)) {
					nextSample = tanPhi * (2 * dt_quarterPeriod - dt);
				}
				if((phase > M_PI) && (phase <= (M_PI + M_PI_2))) {
					nextSample = - (tanPhi * (dt - 2 * dt_quarterPeriod));
				}
				if((phase > (M_PI + M_PI_2)) && (phase <= (2 * M_PI))) {
					nextSample = - (tanPhi * (4 * dt_quarterPeriod - dt));
				}
				buffer[frame] = nextSample * [self amplitudeFlt];		//	the 512 extra multiplications
//				outA[frame] = nextSample * pan;							//	left channel
//				outB[frame] = nextSample * (1.0 - pan);					//	right channel
				phase = phase + phaseIncrement;
				if (phase >= 2.0 * M_PI) {
					phase = phase - 2.0 * M_PI;
				}
			}
			[gAudioToolModelPtr amSetamplitude:[self amplitudeFlt]];
			[gAudioToolModelPtr amSetWaveForm:2];
			break;
		}
	[display addData:buffer samples:samples];
}


@end
